@@ -3,9 +3,10 @@
 
 from __future__ import annotations
 
+import json
 import logging
 import time
-from typing import Any, Optional, Tuple, Type, Union
+from typing import Any, Callable, Iterator, Optional, Tuple, Type, Union
 
 import attrs
 
@@ -14,6 +15,7 @@ from acconeer.exptool.a121._core.entities import (
     Metadata,
     Result,
     SensorConfig,
+    SensorInfo,
     ServerInfo,
     SessionConfig,
 )
@@ -27,6 +29,7 @@ from acconeer.exptool.a121._perf_calc import _PerformanceCalc
 
 from .communication_protocol import CommunicationProtocol
 from .link import BufferedLink
+from .message import AgnosticClientFriends, Message, SystemInfoDict
 from .recorder import Recorder
 
 
@@ -37,27 +40,36 @@ class ClientError(Exception):
     pass
 
 
-class AgnosticClient:
+class AgnosticClient(AgnosticClientFriends):
     _link: BufferedLink
     _default_link_timeout: float
     _link_timeout: float
     _protocol: Type[CommunicationProtocol]
-    _server_info: Optional[ServerInfo]
-    _session_config: Optional[SessionConfig]
-    _metadata: Optional[list[dict[int, Metadata]]]
-    _session_is_started: bool
     _recorder: Optional[Recorder]
     _tick_unwrapper: TickUnwrapper
 
+    # Message friend fields
+    _session_config: Optional[SessionConfig]
+    _session_is_started: bool
+    _metadata: Optional[list[dict[int, Metadata]]]
+    _sensor_infos: dict[int, SensorInfo]
+    _system_info: Optional[SystemInfoDict]
+    _result_queue: list[list[dict[int, Result]]]
+    _message_stream: Iterator[Message]
+
     def __init__(self, link: BufferedLink, protocol: Type[CommunicationProtocol]) -> None:
         self._link = link
         self._protocol = protocol
-        self._server_info = None
+        self._recorder = None
+        self._tick_unwrapper = TickUnwrapper()
+        self._message_stream = iter([])
+
         self._session_config = None
         self._session_is_started = False
         self._metadata = None
-        self._recorder = None
-        self._tick_unwrapper = TickUnwrapper()
+        self._sensor_infos = {}
+        self._system_info = None
+        self._result_queue = []
 
     def _assert_connected(self):
         if not self.connected:
@@ -73,25 +85,92 @@ class AgnosticClient:
         if not self.session_is_started:
             raise ClientError("Session is not started.")
 
+    def _get_message_stream(self) -> Iterator[Message]:
+        """returns an iterator of parsed messages"""
+        while True:
+            header: dict[str, Any] = json.loads(self._link.recv_until(self._protocol.end_sequence))
+            try:
+                payload_size = header["payload_size"]
+            except KeyError:
+                payload = bytes()
+            else:
+                payload = self._link.recv(payload_size)
+
+            resp = self._protocol.parse_message(header, payload)
+            yield resp
+
+    def _assert_deadline_not_reached(self, deadline: Optional[float]) -> None:
+        if deadline is not None and time.time() > deadline:
+            raise ClientError("Client timed out.")
+
+    def _apply_messages_until_message_type_encountered(
+        self, message_type: Type[Message], timeout_s: Optional[float] = None
+    ) -> None:
+        """Retrieves and applies messages until a message of type ``message_type`` is encountered.
+
+        :param message_type: a subclass of ``Message``
+        :param timeout_s: Limit the time spent in this function
+        :raises ClientError:
+            if timeout_s is set and that amount of time has elapsed
+            without predicate evaluating to True
+        """
+        deadline = None if (timeout_s is None) else time.time() + timeout_s
+
+        for message in self._message_stream:
+            message.apply(self)
+
+            if type(message) == message_type:
+                return
+
+            self._assert_deadline_not_reached(deadline)
+
+    def _apply_messages_until(
+        self, predicate: Callable[[], bool], timeout_s: Optional[float] = None
+    ) -> None:
+        """Retrieves and applies messages until `predicate` evaluates to True
+
+        :param predicate: The stop condition
+        :param timeout_s: Limit the time spent in this function
+        :raises ClientError:
+            if timeout_s is set and that amount of time has elapsed
+            without predicate evaluating to True
+        """
+        if predicate():
+            return
+
+        deadline = None if (timeout_s is None) else time.time() + timeout_s
+
+        for message in self._message_stream:
+            # OBS! When iterating self._message_stream, each message is "consumed" from the stream.
+            #      I.e. there is no way to get it back outside this loop.
+            #      This means that we always should ".apply" unless
+            #      we have a really good reason not to.
+            message.apply(self)
+
+            if predicate():
+                return
+
+            self._assert_deadline_not_reached(deadline)
+
     def connect(self) -> None:
         """Connects to the specified host.
 
         :raises: Exception if the host cannot be connected to.
+        :raises: ClientError if server has wrong sensor generation (e.g. "a111")
         """
         self._default_link_timeout = self._link.timeout
         self._link_timeout = self._default_link_timeout
         self._link.connect()
 
-        self._link.send(self._protocol.get_sensor_info_command())
-        sens_response = self._link.recv_until(self._protocol.end_sequence)
-        sensor_infos = self._protocol.get_sensor_info_response(sens_response)
+        self._message_stream = self._get_message_stream()
 
         self._link.send(self._protocol.get_system_info_command())
-        sys_response = self._link.recv_until(self._protocol.end_sequence)
-        self._server_info, sensor = self._protocol.get_system_info_response(
-            sys_response, sensor_infos
-        )
+        self._apply_messages_until(lambda: self._system_info is not None)
 
+        self._link.send(self._protocol.get_sensor_info_command())
+        self._apply_messages_until(lambda: self._sensor_infos != {})
+
+        sensor = self._system_info.get("sensor") if self._system_info else None
         if sensor != "a121":
             self._link.disconnect()
             raise ClientError(f"Wrong sensor version, expected a121 but got {sensor}")
@@ -132,14 +211,14 @@ class AgnosticClient:
             self._link_timeout = self._default_link_timeout
 
         self._link.timeout = self._link_timeout
-
         self._link.send(self._protocol.setup_command(config))
-        reponse_bytes = self._link.recv_until(self._protocol.end_sequence)
-        self._session_config = config
-        self._metadata = self._protocol.setup_response(
-            reponse_bytes, context_session_config=config
-        )
 
+        self._session_config = config
+
+        self._metadata = None
+        self._apply_messages_until(lambda: self._metadata is not None)
+
+        assert self._metadata is not None
         if self.session_config.extended:
             return self._metadata
         else:
@@ -171,9 +250,7 @@ class AgnosticClient:
         self._link.timeout = self._link_timeout
 
         self._link.send(self._protocol.start_streaming_command())
-        reponse_bytes = self._link.recv_until(self._protocol.end_sequence)
-        self._protocol.start_streaming_response(reponse_bytes)
-        self._session_is_started = True
+        self._apply_messages_until(lambda: self.session_is_started)
 
     def get_next(self) -> Union[Result, list[dict[int, Result]]]:
         """Gets results from the server.
@@ -186,15 +263,8 @@ class AgnosticClient:
         """
         self._assert_session_started()
 
-        payload_size, partial_results = self._protocol.get_next_header(
-            bytes_=self._link.recv_until(self._protocol.end_sequence),
-            extended_metadata=self.extended_metadata,
-            ticks_per_second=self.server_info.ticks_per_second,
-        )
-        payload = self._link.recv(payload_size)
-        extended_results = self._protocol.get_next_payload(payload, partial_results)
-
-        extended_results = self._tick_unwrapper.unwrap_ticks(extended_results)
+        self._apply_messages_until(lambda: len(self._result_queue) > 0)
+        extended_results = self._result_queue.pop(0)
 
         if self._recorder is not None:
             self._recorder._sample(extended_results)
@@ -214,58 +284,40 @@ class AgnosticClient:
         """
         self._assert_session_started()
 
-        recorder_result = None
-        if self._recorder is not None:
-            recorder_result = self._recorder._stop()
-            self._recorder = None
+        self._link.send(self._protocol.stop_streaming_command())
 
         try:
-            self._link.send(self._protocol.stop_streaming_command())
-            reponse_bytes = self._drain_buffer(self._link.timeout + 1)
-            self._protocol.stop_streaming_response(reponse_bytes)
+            self._apply_messages_until(
+                lambda: not self.session_is_started, timeout_s=self._link.timeout + 1
+            )
         except Exception:
             raise
         finally:
             self._link.timeout = self._default_link_timeout
-            self._session_is_started = False
             self._tick_unwrapper = TickUnwrapper()
+            self._result_queue = []  # results that are recv:ed post stop_session are discarded
+
+        if self._recorder is None:
+            recorder_result = None
+        else:
+            recorder_result = self._recorder._stop()
+            self._recorder = None
 
         return recorder_result
 
-    def _drain_buffer(
-        self, timeout_s: float = 3.0
-    ) -> bytes:  # TODO: Make `timeout_s` session-dependant
-        """Drains data in the buffer. Returning the first bytes that are not data packets."""
-        start = time.time()
-
-        while time.time() < start + timeout_s:
-            next_header = self._link.recv_until(self._protocol.end_sequence)
-            try:
-                payload_size, _ = self._protocol.get_next_header(
-                    bytes_=next_header,
-                    extended_metadata=self.extended_metadata,
-                    ticks_per_second=self.server_info.ticks_per_second,
-                )
-                _ = self._link.recv(payload_size)
-            except Exception:
-                return next_header
-            else:
-                log.debug("Threw away get_next package when draining buffer")
-        raise ClientError("Client timed out when waiting for 'stop'-response.")
-
     def disconnect(self) -> None:
-        """Disconnects the client from the host.
+        """Disconnects the client from the host."""
 
-        :raises: ``ClientError`` if ``Client`` is not connected.
-        """
         # TODO: Make sure this cleans up corner-cases (like lost connection)
         #       to not hog resources.
-        self._assert_connected()
 
         if self.session_is_started:
             _ = self.stop_session()
 
-        self._server_info = None
+        self._system_info = None
+        self._sensor_infos = {}
+        self._message_stream = iter([])
+        self._metadata = None
         self._link.disconnect()
 
     def __enter__(self):
@@ -278,12 +330,13 @@ class AgnosticClient:
     @property
     def connected(self) -> bool:
         """Whether this Client is connected."""
-        return self._server_info is not None
+
+        return self._sensor_infos != {} and self._system_info is not None
 
     @property
     def session_is_setup(self) -> bool:
         """Whether this Client has a session set up."""
-        return self._session_config is not None
+        return self._metadata is not None
 
     @property
     def session_is_started(self) -> bool:
@@ -295,7 +348,15 @@ class AgnosticClient:
         """The ``ServerInfo``."""
         self._assert_connected()
 
-        return self._server_info  # type: ignore[return-value]
+        assert self._system_info is not None  # Should never happend if client is connected
+        return ServerInfo(
+            rss_version=self._system_info["rss_version"],
+            sensor_count=self._system_info["sensor_count"],
+            ticks_per_second=self._system_info["ticks_per_second"],
+            hardware_name=self._system_info.get("hw", None),
+            sensor_infos=self._sensor_infos,
+            max_baudrate=self._system_info.get("max_baudrate"),
+        )
 
     @property
     def client_info(self) -> ClientInfo:
