@@ -24,7 +24,6 @@ from ._aggregator import (
     ProcessorSpec,
 )
 from ._processors import (
-    DEFAULT_CFAR_ONE_SIDED,
     DEFAULT_FIXED_THRESHOLD_VALUE,
     DEFAULT_THRESHOLD_SENSITIVITY,
     MeasurementType,
@@ -68,7 +67,7 @@ Plan = Dict[MeasurementType, List[SubsweepGroupPlan]]
 
 
 @attrs.mutable(kw_only=True)
-class DetectorContext:
+class DetectorContext(AlgoConfigBase):
     offset_m: Optional[float] = attrs.field(default=None)
     direct_leakage: Optional[npt.NDArray[np.complex_]] = attrs.field(default=None)
     phase_jitter_comp_reference: Optional[npt.NDArray[np.float_]] = attrs.field(default=None)
@@ -88,10 +87,10 @@ class DetectorContext:
 
 @attrs.mutable(kw_only=True)
 class DetectorConfig(AlgoConfigBase):
-    start_m: float = attrs.field(default=0.2)
+    start_m: float = attrs.field(default=0.25)
     """Start point of measurement interval in meters."""
 
-    end_m: float = attrs.field(default=1.0)
+    end_m: float = attrs.field(default=3.0)
     """End point of measurement interval in meters."""
 
     max_step_length: Optional[int] = attrs.field(default=None)  # TODO: Check validity
@@ -113,7 +112,7 @@ class DetectorConfig(AlgoConfigBase):
     """Threshold method"""
 
     peaksorting_method: PeakSortingMethod = attrs.field(
-        default=PeakSortingMethod.STRONGEST,
+        default=PeakSortingMethod.HIGHEST_RCS,
         converter=PeakSortingMethod,
     )
     """Sorting method of estimated distances."""
@@ -128,9 +127,6 @@ class DetectorConfig(AlgoConfigBase):
     """Sensitivity of threshold. High sensitivity equals low detection threshold, low sensitivity
     equals high detection threshold."""
 
-    cfar_one_sided: bool = attrs.field(default=DEFAULT_CFAR_ONE_SIDED)
-    """Use one sided CFAR threshold."""
-
 
 @attrs.frozen(kw_only=True)
 class DetectorResult:
@@ -169,13 +165,6 @@ class Detector:
     MAX_HWAAS = 511
     MIN_HWAAS = 1
     HWAAS_MIN_DISTANCE = 1.0
-    RLG_PER_HWAAS_MAP = {
-        a121.Profile.PROFILE_1: 11.3,
-        a121.Profile.PROFILE_2: 13.7,
-        a121.Profile.PROFILE_3: 19.0,
-        a121.Profile.PROFILE_4: 20.5,
-        a121.Profile.PROFILE_5: 21.6,
-    }
 
     session_config: a121.SessionConfig
     processor_specs: List[ProcessorSpec]
@@ -212,6 +201,12 @@ class Detector:
             raise ValueError("Session config not defined")
 
     def calibrate_close_range(self) -> None:
+        """Calibrates the close range measurement parameters used when subtracting the direct
+        leakage from the measured signal.
+
+        The parameters calibrated are the direct leakage and a phase reference, used to reduce
+        the amount of phase jitter, with the purpose of reducing the residual.
+        """
         self._validate_ready_for_calibration()
 
         close_range_spec = self._filter_close_range_spec(self.processor_specs)
@@ -243,6 +238,8 @@ class Detector:
         self.context.recorded_thresholds_noise_std = None
 
     def record_threshold(self) -> None:
+        """Calibrates the parameters used when forming the recorded threshold."""
+
         self._validate_ready_for_calibration()
 
         # TODO: Ignore/override threshold method while recording threshold
@@ -331,7 +328,7 @@ class Detector:
         for spec in self.processor_specs:
             bg_noise_std_in_subsweep = []
             result = extended_result[spec.group_index][spec.sensor_id]
-            sensor_config = session_config.groups[spec.group_index][spec.sensor_id]
+            sensor_config = self.session_config.groups[spec.group_index][spec.sensor_id]
             subsweep_configs = sensor_config.subsweeps
             for idx in spec.subsweep_indexes:
                 if not subsweep_configs[idx].enable_loopback:
@@ -342,6 +339,8 @@ class Detector:
         self.context.bg_noise_std = bg_noise_std
 
     def calibrate_offset(self) -> None:
+        """Estimates sensor offset error based on loopback measurement."""
+
         self._validate_ready_for_calibration()
 
         sensor_config = a121.SensorConfig(
@@ -369,6 +368,8 @@ class Detector:
     def get_detector_status(
         cls, config: DetectorConfig, context: DetectorContext, sensor_id: int
     ) -> DetectorStatus:
+        """Returns the detector status along with the detector state."""
+
         if not cls._valid_detector_config_range(config=config):
             return DetectorStatus(
                 detector_state=DetailedStatus.INVALID_DETECTOR_CONFIG_RANGE,
@@ -463,6 +464,8 @@ class Detector:
     def start(
         self, recorder: Optional[a121.Recorder] = None, skip_calibration: bool = False
     ) -> None:
+        """Method for setting up measurement session."""
+
         if self.started:
             raise RuntimeError("Already started")
 
@@ -508,6 +511,7 @@ class Detector:
         self.started = True
 
     def get_next(self) -> DetectorResult:
+        """Called from host to get next measurement."""
         if not self.started:
             raise RuntimeError("Not started")
 
@@ -525,6 +529,8 @@ class Detector:
         )
 
     def update_config(self, config: DetectorConfig) -> None:
+        """Updates the session config and processor specification based on the detector
+        configuration."""
         (
             self.session_config,
             self.processor_specs,
@@ -533,6 +539,7 @@ class Detector:
         )
 
     def stop(self) -> Any:
+        """Stops the measurement session."""
         if not self.started:
             raise RuntimeError("Already stopped")
 
@@ -584,7 +591,6 @@ class Detector:
                 threshold_method=config.threshold_method,
                 fixed_threshold_value=config.fixed_threshold_value,
                 threshold_sensitivity=config.threshold_sensitivity,
-                cfar_one_sided=config.cfar_one_sided,
             )
 
             for subsweep_indexes in processor_specs_subsweep_indexes:
@@ -606,115 +612,48 @@ class Detector:
         """
         Create dictionary containing group plans for close and far range measurements.
 
-        Constants used:
-        - MIN_DIST_M defines the shortest distance possible to measure free of leakage.
-
-        Outline of logic:
-        - If the start is closer than the transition point, a close range subsweep plan is added.
-        - If the end is further away than the transition point, far range subsweep plans are added.
-        - max_profile is used in the far range region to achive high SNR and low power consumption.
-        - A shorter profile is used between the transition point and the MIN_DIST_M of max_profile.
+        - Close range measurement: Add Subsweep group if the user defined starting point is
+        effected by the direct leakage.
+        - Transition region: Add group plans to bridge the gap between the start of the far range
+        measurement region(either end of close range region or user defined start_m) and the
+        shortest measurable distance with max_profile, free from direct leakage interference.
+        - Add group plan with max_profile. Increase HWAAS as a function of distance to maintain
+        SNR throughout the sweep.
         """
-
-        min_dist_m = cls._add_to_min_dist_m(config)
-        transition_m = list(min_dist_m.values())[0]
-
         plans = {}
 
-        if config.start_m < transition_m:
-            profile = a121.Profile.PROFILE_1
-            step_length = cls._limit_step_length(profile, config.max_step_length)
-            breakpoints = cls._m_to_points([config.start_m, transition_m], step_length)
-            hwaas = cls._calculate_hwaas(profile, breakpoints, config.signal_quality, step_length)
+        # Determine shortest direct leakage free distance per profile
+        min_dist_m = cls._calc_leakage_free_min_dist(config)
 
-            has_neighbour = (False, transition_m < config.end_m)
-            extended_breakpoints = cls._add_margin_to_breakpoints(
-                profile=profile,
-                step_length=step_length,
-                base_bpts=breakpoints,
-                has_neighbour=has_neighbour,
-                config=config,
-                is_close_range_measurement=True,
+        close_range_transition_m = min_dist_m[a121.Profile.PROFILE_1]
+
+        # Add close range group plan if applicable
+        if config.start_m < close_range_transition_m:
+            plans[MeasurementType.CLOSE_RANGE] = cls._get_close_range_group_plan(
+                close_range_transition_m, config
             )
 
-            plans[MeasurementType.CLOSE_RANGE] = [
-                SubsweepGroupPlan(
-                    step_length=step_length,
-                    breakpoints=extended_breakpoints,
-                    profile=profile,
-                    hwaas=hwaas,
-                )
-            ]
+        # Define transition group plans
+        transition_subgroup_plans = cls._get_transition_group_plans(
+            config, min_dist_m, MeasurementType.CLOSE_RANGE in plans
+        )
 
-        far_subgroup_plans = []
-        far_range_start_m = np.max([config.start_m, transition_m])
-        if (
-            config.max_profile is not a121.Profile.PROFILE_1
-            and config.start_m < min_dist_m[config.max_profile]
-            and far_range_start_m < config.end_m
-        ):
-            min_dists_m = np.array(list(min_dist_m.values()))
-            min_dists_profiles = np.array(list(min_dist_m.keys()))
-            (viable_profile_idx,) = np.where(min_dists_m <= far_range_start_m)
-            profile_to_be_used = min_dists_profiles[viable_profile_idx[-1]]
+        # The number of available subsweeps in the group with max profile.
+        num_remaining_subsweeps = cls.NUM_SUBSWEEPS_IN_SENSOR_CONFIG - len(
+            transition_subgroup_plans
+        )
 
-            end_m = min(config.end_m, min_dist_m[config.max_profile])
-            step_length = cls._limit_step_length(profile_to_be_used, config.max_step_length)
-            breakpoints = cls._m_to_points([far_range_start_m, end_m], step_length)
-            hwaas = cls._calculate_hwaas(
-                profile_to_be_used, breakpoints, config.signal_quality, step_length
-            )
+        # No neighbours if no close range measurement or transition groups defined.
+        has_neighbouring_subsweep = (
+            MeasurementType.CLOSE_RANGE in plans or len(transition_subgroup_plans) != 0
+        )
 
-            has_neighbour = (len(plans) != 0, min_dist_m[config.max_profile] < end_m)
-            extended_breakpoints = cls._add_margin_to_breakpoints(
-                profile=profile_to_be_used,
-                step_length=step_length,
-                base_bpts=breakpoints,
-                has_neighbour=has_neighbour,
-                config=config,
-                is_close_range_measurement=False,
-            )
+        # Define group plans with max profile
+        max_profile_subgroup_plans = cls._get_max_profile_group_plans(
+            config, min_dist_m, has_neighbouring_subsweep, num_remaining_subsweeps
+        )
 
-            far_subgroup_plans.append(
-                SubsweepGroupPlan(
-                    step_length=step_length,
-                    breakpoints=extended_breakpoints,
-                    profile=profile_to_be_used,
-                    hwaas=hwaas,
-                )
-            )
-
-        if min_dist_m[config.max_profile] < config.end_m:
-            subsweep_start_m = max([config.start_m, min_dist_m[config.max_profile]])
-            breakpoints_m = np.linspace(
-                subsweep_start_m,
-                config.end_m,
-                cls.NUM_SUBSWEEPS_IN_SENSOR_CONFIG + 1 - len(far_subgroup_plans),
-            ).tolist()
-
-            profile = config.max_profile
-            step_length = cls._limit_step_length(config.max_profile, config.max_step_length)
-            breakpoints = cls._m_to_points(breakpoints_m, step_length)
-            hwaas = cls._calculate_hwaas(profile, breakpoints, config.signal_quality, step_length)
-
-            has_neighbour = (len(plans) != 0 or len(far_subgroup_plans) != 0, False)
-            extended_breakpoints = cls._add_margin_to_breakpoints(
-                profile=profile,
-                step_length=step_length,
-                base_bpts=breakpoints,
-                has_neighbour=has_neighbour,
-                config=config,
-                is_close_range_measurement=False,
-            )
-
-            far_subgroup_plans.append(
-                SubsweepGroupPlan(
-                    step_length=step_length,
-                    breakpoints=extended_breakpoints,
-                    profile=profile,
-                    hwaas=hwaas,
-                )
-            )
+        far_subgroup_plans = transition_subgroup_plans + max_profile_subgroup_plans
 
         if len(far_subgroup_plans) != 0:
             plans[MeasurementType.FAR_RANGE] = far_subgroup_plans
@@ -722,7 +661,136 @@ class Detector:
         return plans
 
     @classmethod
-    def _add_to_min_dist_m(cls, config: DetectorConfig) -> Dict[a121.Profile, float]:
+    def _get_close_range_group_plan(
+        cls, transition_m: float, config: DetectorConfig
+    ) -> list[SubsweepGroupPlan]:
+        """Define the group plan for close range measurements.
+
+        The close range measurement always use profile 1 to minimize direct leakage region.
+        """
+        profile = a121.Profile.PROFILE_1
+        # No left neighbour as this is the first subsweep when close range measurement is
+        # applicable.
+        has_neighbour = (False, transition_m < config.end_m)
+        return [
+            cls._create_group_plan(
+                profile, config, [config.start_m, transition_m], has_neighbour, True
+            )
+        ]
+
+    @classmethod
+    def _get_transition_group_plans(
+        cls,
+        config: DetectorConfig,
+        min_dist_m: Dict[a121.Profile, float],
+        has_close_range_measurement: bool,
+    ) -> list[SubsweepGroupPlan]:
+        """Define the transition segment group plans.
+
+        The purpose of the transition group is to bridge the gap between the start point of the
+        far measurement region and the point where max_profile can be used without interference
+        of direct leakage.
+
+        The transition region can consist of maximum two subsweeps, where the first utilize profile
+        1 and the second profile 3. Whether both, one or none is used depends on the user provided
+        detector config.
+        """
+        transition_profiles = [
+            profile
+            for profile in [a121.Profile.PROFILE_1, a121.Profile.PROFILE_3]
+            if profile.value < config.max_profile.value
+        ]
+        transition_profiles.append(config.max_profile)
+
+        transition_subgroup_plans: list = []
+
+        for i in range(len(transition_profiles) - 1):
+            profile = transition_profiles[i]
+            next_profile = transition_profiles[i + 1]
+
+            if config.start_m < min_dist_m[next_profile] and min_dist_m[profile] < config.end_m:
+                start_m = max(min_dist_m[profile], config.start_m)
+                end_m = min(config.end_m, min_dist_m[next_profile])
+                has_neighbour = (
+                    has_close_range_measurement or len(transition_subgroup_plans) != 0,
+                    min_dist_m[next_profile] < end_m,
+                )
+
+                transition_subgroup_plans.append(
+                    cls._create_group_plan(profile, config, [start_m, end_m], has_neighbour, False)
+                )
+
+        return transition_subgroup_plans
+
+    @classmethod
+    def _get_max_profile_group_plans(
+        cls,
+        config: DetectorConfig,
+        min_dist_m: Dict[a121.Profile, float],
+        has_neighbouring_subsweep: bool,
+        num_remaining_subsweeps: int,
+    ) -> list[SubsweepGroupPlan]:
+        """Define far range group plans with max_profile
+
+        Divide the measurement range from the shortest leakage free distance of max_profile to
+        the end point into equidistance segments and assign HWAAS according to the radar equation
+        to maintain SNR throughout the sweep.
+        """
+
+        if min_dist_m[config.max_profile] < config.end_m:
+            subsweep_start_m = max([config.start_m, min_dist_m[config.max_profile]])
+            breakpoints_m = np.linspace(
+                subsweep_start_m,
+                config.end_m,
+                num_remaining_subsweeps + 1,
+            ).tolist()
+
+            return [
+                cls._create_group_plan(
+                    config.max_profile,
+                    config,
+                    breakpoints_m,
+                    (has_neighbouring_subsweep, False),
+                    False,
+                )
+            ]
+        else:
+            return []
+
+    @classmethod
+    def _create_group_plan(
+        cls,
+        profile: a121.Profile,
+        config: DetectorConfig,
+        breakpoints_m: list[float],
+        has_neighbour: Tuple[bool, bool],
+        is_close_range_measurement: bool,
+    ) -> SubsweepGroupPlan:
+        """Creates a group plan."""
+        step_length = cls._limit_step_length(profile, config.max_step_length)
+        breakpoints = cls._m_to_points(breakpoints_m, step_length)
+        hwaas = cls._calculate_hwaas(profile, breakpoints, config.signal_quality, step_length)
+
+        extended_breakpoints = cls._add_margin_to_breakpoints(
+            profile=profile,
+            step_length=step_length,
+            base_bpts=breakpoints,
+            has_neighbour=has_neighbour,
+            config=config,
+            is_close_range_measurement=is_close_range_measurement,
+        )
+
+        return SubsweepGroupPlan(
+            step_length=step_length,
+            breakpoints=extended_breakpoints,
+            profile=profile,
+            hwaas=hwaas,
+        )
+
+    @classmethod
+    def _calc_leakage_free_min_dist(cls, config: DetectorConfig) -> Dict[a121.Profile, float]:
+        """This function calculates the shortest leakage free distance per profile, for all profiles
+        up to max_profile"""
         min_dist_m = {}
         for profile, min_dist in cls.MIN_LEAKAGE_FREE_DIST_M.items():
             min_dist_m[profile] = min_dist
@@ -734,16 +802,21 @@ class Detector:
                     * Processor.APPROX_BASE_STEP_LENGTH_M
                 )
                 min_dist_m[profile] += cfar_margin_m
+
+            if profile == config.max_profile:
+                # All profiles up to max_profile has been added. Break and return result.
+                break
+
         return min_dist_m
 
     @classmethod
     def _calculate_hwaas(
         cls, profile: a121.Profile, breakpoints: list[int], signal_quality: float, step_length: int
     ) -> list[int]:
-        rlg_per_hwaas = cls.RLG_PER_HWAAS_MAP[profile]
+        rlg_per_hwaas = Aggregator.RLG_PER_HWAAS_MAP[profile]
         hwaas = []
         for idx in range(len(breakpoints) - 1):
-            processing_gain = cls._calc_processing_gain(profile, step_length)
+            processing_gain = Aggregator.calc_processing_gain(profile, step_length)
             subsweep_end_point_m = max(
                 Processor.APPROX_BASE_STEP_LENGTH_M * breakpoints[idx + 1],
                 cls.HWAAS_MIN_DISTANCE,
@@ -753,21 +826,6 @@ class Detector:
             hwaas.append(np.clip(hwaas_in_subsweep, cls.MIN_HWAAS, cls.MAX_HWAAS))
         return hwaas
 
-    @staticmethod
-    def _calc_processing_gain(profile: a121.Profile, step_length: int) -> float:
-        envelope_base_length_m = Processor.ENVELOPE_FWHM_M[profile] * 2  # approx envelope width
-        num_points_in_envelope = int(
-            envelope_base_length_m / (step_length * Processor.APPROX_BASE_STEP_LENGTH_M)
-        )
-        mid_point = num_points_in_envelope // 2
-        pulse = np.concatenate(
-            (
-                np.linspace(0, 1, mid_point),
-                np.linspace(1, 0, num_points_in_envelope - mid_point),
-            )
-        )
-        return float(np.sum(pulse**2))
-
     @classmethod
     def _add_margin_to_breakpoints(
         cls,
