@@ -4,7 +4,6 @@
 from __future__ import annotations
 
 import logging
-import pickle
 from pathlib import Path
 from typing import Any, Callable, Optional
 
@@ -19,6 +18,7 @@ import pyqtgraph as pg
 
 import acconeer.exptool as et
 from acconeer.exptool import a121
+from acconeer.exptool.a121.algo._base import AlgoConfigBase
 from acconeer.exptool.a121.algo._plugins import (
     DetectorBackendPluginBase,
     DetectorPlotPluginBase,
@@ -53,16 +53,33 @@ from ._detector import Detector, DetectorConfig, DetectorResult, _load_algo_data
 log = logging.getLogger(__name__)
 
 
+@attrs.mutable(kw_only=True)
+class PlotConfig(AlgoConfigBase):
+    number_of_zones: Optional[int] = attrs.field(default=None)
+
+
 @attrs.mutable(kw_only=True)
 class SharedState:
     sensor_id: int = attrs.field(default=1)
     config: DetectorConfig = attrs.field(factory=DetectorConfig)
+    plot_config: PlotConfig = attrs.field(factory=PlotConfig)
     replaying: bool = attrs.field(default=False)
 
 
 @attrs.frozen(kw_only=True)
-class Save:
+class Save(AlgoConfigBase):
     config: DetectorConfig = attrs.field()
+    plot_config: PlotConfig = attrs.field()
+
+    def to_dict(self) -> dict[str, Any]:
+        return {"config": self.config.to_dict(), "plot_config": self.plot_config.to_dict()}
+
+    @classmethod
+    def from_dict(cls: type[Save], obj: dict[str, Any]) -> Save:
+        return Save(
+            config=DetectorConfig.from_dict(obj["config"]),
+            plot_config=PlotConfig.from_dict(obj["plot_config"]),
+        )
 
 
 class BackendPlugin(DetectorBackendPluginBase[SharedState]):
@@ -79,27 +96,17 @@ class BackendPlugin(DetectorBackendPluginBase[SharedState]):
         self.restore_defaults()
 
     @is_task
-    def deserialize(self, *, data: bytes) -> None:
-        try:
-            obj = pickle.loads(data)
-        except Exception:
-            log.warning("Could not load pickled - pickle.loads() failed")
-            return
-
-        if not isinstance(obj, Save):
-            log.warning("Could not load pickled - not the correct type")
-            return
-
-        if not isinstance(obj.config, DetectorConfig):
-            log.warning("Could not load pickled - not the correct type")
-            return
-
+    def _from_cache(self, *, data: dict) -> None:
+        obj = Save.from_dict(data)
         self.shared_state.config = obj.config
+        self.shared_state.plot_config = obj.plot_config
         self.broadcast(sync=True)
 
-    def _serialize(self) -> bytes:
-        obj = Save(config=self.shared_state.config)
-        return pickle.dumps(obj, protocol=4)
+    def _to_cache(self) -> dict:
+        return Save(
+            config=self.shared_state.config,
+            plot_config=self.shared_state.plot_config,
+        ).to_dict()
 
     def broadcast(self, sync: bool = False) -> None:
         super().broadcast()
@@ -117,6 +124,11 @@ class BackendPlugin(DetectorBackendPluginBase[SharedState]):
         self.shared_state.sensor_id = sensor_id
         self.broadcast(sync=True)
 
+    @is_task
+    def update_plot_config(self, *, config: PlotConfig) -> None:
+        self.shared_state.plot_config = config
+        self.broadcast(sync=True)
+
     @property
     def _client(self) -> Optional[a121.Client]:
         if self._replaying_client is not None:
@@ -149,7 +161,7 @@ class BackendPlugin(DetectorBackendPluginBase[SharedState]):
                 kwargs={
                     "generation": PluginGeneration.A121,
                     "key": self.key,
-                    "data": self._serialize(),
+                    "data": self._to_cache(),
                 },
             )
         )
@@ -222,6 +234,7 @@ class BackendPlugin(DetectorBackendPluginBase[SharedState]):
                 kwargs=dict(
                     detector_config=self.shared_state.config,
                     sensor_config=Detector._get_sensor_config(self.shared_state.config),
+                    plot_config=self.shared_state.plot_config,
                 ),
                 recipient="plot_plugin",
             )
@@ -300,7 +313,12 @@ class PlotPlugin(DetectorPlotPluginBase):
         assert isinstance(message.data, DetectorResult)
         self.update(message.data)
 
-    def setup(self, detector_config: DetectorConfig, sensor_config: a121.SensorConfig) -> None:
+    def setup(
+        self,
+        detector_config: DetectorConfig,
+        sensor_config: a121.SensorConfig,
+        plot_config: PlotConfig,
+    ) -> None:
         self.detector_config = detector_config
         self.distances = np.linspace(
             detector_config.start_m, detector_config.end_m, sensor_config.num_points
@@ -308,11 +326,15 @@ class PlotPlugin(DetectorPlotPluginBase):
 
         self.history_length_s = 10
 
-        max_num_of_sectors = max(6, self.distances.size // 3)
-        self.sector_size = max(1, -(-self.distances.size // max_num_of_sectors))
-        self.num_sectors = -(-self.distances.size // self.sector_size)
-        self.sector_offset = (self.num_sectors * self.sector_size - self.distances.size) // 2
+        if plot_config.number_of_zones:
+            self.num_sectors = min(plot_config.number_of_zones, self.distances.size)
+            self.sector_size = max(1, -(-self.distances.size // self.num_sectors))
+        else:
+            max_num_of_sectors = max(6, self.distances.size // 3)
+            self.sector_size = max(1, -(-self.distances.size // max_num_of_sectors))
+            self.num_sectors = -(-self.distances.size // self.sector_size)
 
+        self.sector_offset = (self.num_sectors * self.sector_size - self.distances.size) // 2
         win = self.plot_layout
 
         self.intra_limit_lines = []
@@ -622,6 +644,21 @@ class ViewPlugin(DetectorViewPluginBase):
         self.config_editor.sig_update.connect(self._on_config_update)
         scrolly_layout.addWidget(self.config_editor)
 
+        self.plot_config_editor = AttrsConfigEditor[PlotConfig](
+            title="Plot parameters",
+            factory_mapping={
+                "number_of_zones": pidgets.OptionalIntParameterWidgetFactory(
+                    name_label_text="Detection zones",
+                    checkbox_label_text="Override",
+                    limits=(1, 10),
+                    init_set_value=3,
+                )
+            },
+            parent=self.scrolly_widget,
+        )
+        self.plot_config_editor.sig_update.connect(self._on_plot_config_update)
+        scrolly_layout.addWidget(self.plot_config_editor)
+
         self.sticky_widget.setLayout(sticky_layout)
         self.scrolly_widget.setLayout(scrolly_layout)
 
@@ -638,6 +675,18 @@ class ViewPlugin(DetectorViewPluginBase):
                 suffix=" m",
                 decimals=3,
             ),
+            "profile": pidgets.OptionalEnumParameterWidgetFactory(
+                name_label_text="Profile",
+                checkbox_label_text="Override",
+                enum_type=a121.Profile,
+                label_mapping={
+                    a121.Profile.PROFILE_1: "1 (shortest)",
+                    a121.Profile.PROFILE_2: "2",
+                    a121.Profile.PROFILE_3: "3",
+                    a121.Profile.PROFILE_4: "4",
+                    a121.Profile.PROFILE_5: "5 (longest)",
+                },
+            ),
             "step_length": pidgets.OptionalIntParameterWidgetFactory(
                 name_label_text="Step length",
                 checkbox_label_text="Override",
@@ -666,14 +715,65 @@ class ViewPlugin(DetectorViewPluginBase):
                 decimals=2,
                 limits=(0, 5),
             ),
+            "intra_frame_time_const": pidgets.FloatSliderParameterWidgetFactory(
+                name_label_text="Intra time constant",
+                suffix=" s",
+                decimals=2,
+                limits=(0, 1),
+            ),
+            "intra_output_time_const": pidgets.FloatSliderParameterWidgetFactory(
+                name_label_text="Intra output time constant",
+                suffix=" s",
+                decimals=2,
+                limits=(0.01, 20),
+                log_scale=True,
+            ),
             "inter_enable": pidgets.CheckboxParameterWidgetFactory(
                 name_label_text="Enable slow motion detection"
             ),
+            "inter_phase_boost": pidgets.CheckboxParameterWidgetFactory(
+                name_label_text="Enable phase boost"
+            ),
             "inter_detection_threshold": pidgets.FloatSliderParameterWidgetFactory(
                 name_label_text="Inter detection threshold",
                 decimals=2,
                 limits=(0, 5),
             ),
+            "inter_frame_fast_cutoff": pidgets.FloatSliderParameterWidgetFactory(
+                name_label_text="Inter fast cutoff freq.",
+                suffix=" Hz",
+                decimals=2,
+                limits=(1, 50),
+                log_scale=True,
+            ),
+            "inter_frame_slow_cutoff": pidgets.FloatSliderParameterWidgetFactory(
+                name_label_text="Inter slow cutoff freq.",
+                suffix=" Hz",
+                decimals=2,
+                limits=(0.01, 1),
+                log_scale=True,
+            ),
+            "inter_frame_deviation_time_const": pidgets.FloatSliderParameterWidgetFactory(
+                name_label_text="Inter time constant",
+                suffix=" s",
+                decimals=2,
+                limits=(0.01, 20),
+                log_scale=True,
+            ),
+            "inter_output_time_const": pidgets.FloatSliderParameterWidgetFactory(
+                name_label_text="Inter output time constant",
+                suffix=" s",
+                decimals=2,
+                limits=(0.01, 20),
+                log_scale=True,
+            ),
+            "inter_frame_presence_timeout": pidgets.OptionalIntParameterWidgetFactory(
+                name_label_text="Presence timeout",
+                checkbox_label_text="Enable",
+                suffix=" s",
+                limits=(1, 30),
+                init_set_value=5,
+            ),
         }
 
     def on_app_model_update(self, app_model: AppModel) -> None:
@@ -687,6 +787,8 @@ class ViewPlugin(DetectorViewPluginBase):
 
             self.config_editor.set_data(None)
             self.config_editor.setEnabled(False)
+            self.plot_config_editor.set_data(None)
+            self.plot_config_editor.setEnabled(False)
             self.sensor_id_pidget.set_parameter(None)
 
             return
@@ -697,6 +799,8 @@ class ViewPlugin(DetectorViewPluginBase):
 
         self.config_editor.setEnabled(app_model.plugin_state == PluginState.LOADED_IDLE)
         self.config_editor.set_data(state.config)
+        self.plot_config_editor.setEnabled(app_model.plugin_state == PluginState.LOADED_IDLE)
+        self.plot_config_editor.set_data(state.plot_config)
         self.sensor_id_pidget.set_parameter(state.sensor_id)
         self.sensor_id_pidget.setEnabled(app_model.plugin_state.is_steady)
 
@@ -712,6 +816,9 @@ class ViewPlugin(DetectorViewPluginBase):
     def _on_config_update(self, config: DetectorConfig) -> None:
         self.app_model.put_backend_plugin_task("update_config", {"config": config})
 
+    def _on_plot_config_update(self, config: PlotConfig) -> None:
+        self.app_model.put_backend_plugin_task("update_plot_config", {"config": config})
+
     # TODO: move to detector base (?)
     def handle_message(self, message: GeneralMessage) -> None:
         if message.name == "sync":
@@ -757,6 +864,6 @@ PRESENCE_DETECTOR_PLUGIN = PluginSpec(
     generation=PluginGeneration.A121,
     key="presence_detector",
     title="Presence detector",
-    # description="",
+    description="Detect human presence.",
     family=PluginFamily.DETECTOR,
 )
