@@ -22,6 +22,9 @@ class ProcessorConfig(AlgoConfigBase):
     inter_enable: bool = attrs.field(default=True)
     intra_detection_threshold: float = attrs.field(default=1.3)
     inter_detection_threshold: float = attrs.field(default=1)
+    inter_phase_boost: bool = attrs.field(default=False)
+    phase_adaptivity_tc: float = attrs.field(default=5)
+    inter_frame_presence_timeout: Optional[int] = attrs.field(default=None)
     inter_frame_fast_cutoff: float = attrs.field(default=20.0)
     inter_frame_slow_cutoff: float = attrs.field(default=0.2)
     inter_frame_deviation_time_const: float = attrs.field(default=0.5)
@@ -73,6 +76,7 @@ class Processor(ProcessorBase[ProcessorConfig, ProcessorResult]):
     }
 
     APPROX_BASE_STEP_LENGTH_M = 2.5e-3
+    MAX_AMPLITUDE_WEIGHT = 15
 
     def __init__(
         self,
@@ -120,6 +124,17 @@ class Processor(ProcessorBase[ProcessorConfig, ProcessorResult]):
         nd = self.noise_est_diff_order
         self.noise_norm_factor = np.sqrt(np.sum(np.square(binom(nd, np.arange(nd + 1)))))
 
+        self.lp_mean_sweep_for_abs = np.zeros(self.num_distances, dtype=np.complex_)
+        self.lp_mean_sweep_for_phase = np.zeros(self.num_distances, dtype=np.complex_)
+        self.mean_sweep_tc = self.processor_config.phase_adaptivity_tc
+        self.mean_sweep_sf = self._tc_to_sf(self.mean_sweep_tc, self.f)
+        self.lp_phase_shift = np.zeros(self.num_distances)
+        self.inter_phase_boost = self.processor_config.inter_phase_boost
+
+        self.inter_frame_presence_timeout = processor_config.inter_frame_presence_timeout
+        self.previous_presence_score = 0
+        self.negative_count = 0
+
         self.fast_lp_mean_sweep = np.zeros(self.num_distances)
         self.slow_lp_mean_sweep = np.zeros(self.num_distances)
         self.lp_inter_dev = np.zeros(self.num_distances)
@@ -159,6 +174,11 @@ class Processor(ProcessorBase[ProcessorConfig, ProcessorResult]):
         self.intra_output_sf = self._tc_to_sf(processor_config.intra_output_time_const, self.f)
         self.inter_output_sf = self._tc_to_sf(processor_config.inter_output_time_const, self.f)
 
+        self.inter_frame_presence_timeout = self.processor_config.inter_frame_presence_timeout
+        self.inter_phase_boost = self.processor_config.inter_phase_boost
+        self.mean_sweep_tc = processor_config.phase_adaptivity_tc
+        self.mean_sweep_sf = self._tc_to_sf(self.mean_sweep_tc, self.f)
+
     @staticmethod
     def _cutoff_to_sf(fc: float, fs: float) -> float:
         """Cutoff frequency to smoothing factor conversion"""
@@ -216,6 +236,81 @@ class Processor(ProcessorBase[ProcessorConfig, ProcessorResult]):
             a = np.pad(a, pad_width, "constant")
             return np.correlate(a, b, mode="same")[pad_width:-pad_width]
 
+    @staticmethod
+    def _calculate_phase_shift(a: npt.NDArray, b: npt.NDArray) -> npt.NDArray[np.float_]:
+        phase_a = np.angle(a)
+        phase_b = np.angle(b)
+        phases_unwrapped = np.unwrap([phase_a, phase_b], axis=0)
+        phase_shift = np.abs(phases_unwrapped[0, :] - phases_unwrapped[1, :])
+
+        return phase_shift  # type: ignore[no-any-return]
+
+    def _calculate_phase_and_amp_weight(self, mean_sweep: npt.NDArray) -> np.float_:
+        """
+        Calculation of a weight factor based on phase shift and amplitude.
+        The phase shift between the mean sweep and a lp-filtered mean sweep is
+        calculated to amplify slow motions.
+        The amplitude of the mean sweep is multiplied with the phase shift to reduce
+        noise amplification.
+        Before multiplication with the phase shift, the amplitude is truncated to reduce
+        side effects from very strong reflective objects.
+        """
+
+        phase_shift = self._calculate_phase_shift(self.lp_mean_sweep_for_phase, mean_sweep)
+        sf = self._dynamic_sf(self.inter_dev_sf, self.update_index)
+        self.lp_phase_shift = sf * self.lp_phase_shift + (1.0 - sf) * phase_shift
+
+        self.lp_mean_sweep_for_abs = sf * self.lp_mean_sweep_for_abs + (1.0 - sf) * mean_sweep
+        abs_lp_mean_sweep = np.abs(self.lp_mean_sweep_for_abs)
+
+        norm_abs_mean_sweep = np.divide(
+            abs_lp_mean_sweep,
+            self.lp_noise,
+            out=np.zeros_like(abs_lp_mean_sweep),
+            where=(self.lp_noise > 1.0),
+        )
+
+        norm_abs_mean_sweep *= np.sqrt(self.sweeps_per_frame)
+
+        # Truncate if amplitude is too strong
+        norm_abs_mean_sweep = np.minimum(
+            norm_abs_mean_sweep, np.ones(norm_abs_mean_sweep.shape[0]) * self.MAX_AMPLITUDE_WEIGHT
+        )
+
+        return self.lp_phase_shift * norm_abs_mean_sweep  # type: ignore[no-any-return]
+
+    def _inter_presence_score_scaling(self) -> None:
+        """
+        Scaling of self.inter_presence_score for faster decline when loosing detection.
+        Start exponential scaling after self.inter_frame_presence_timeout seconds.
+        """
+
+        if self.inter_frame_presence_timeout is None:
+            raise ValueError("inter_frame_presence_timeout must be set")
+
+        scaling_factor = np.exp(
+            np.maximum(self.negative_count - self.inter_frame_presence_timeout * self.f, 0)
+            / (self.inter_frame_presence_timeout * self.f)
+        )
+        self.inter_presence_score /= scaling_factor
+
+    def _mean_sweep_sf_scaling(self) -> None:
+        """
+        Scaling of self.mean_sweep_sf for faster adaptation to the environment when
+        loosing detection.
+        Start exponential scaling after self.inter_frame_presence_timeout seconds.
+        """
+
+        if self.inter_frame_presence_timeout is None:
+            raise ValueError("inter_frame_presence_timeout must be set")
+
+        scaling_factor = np.exp(
+            np.maximum(self.negative_count - self.inter_frame_presence_timeout * self.f, 0)
+            * self.mean_sweep_tc
+            / self.inter_frame_presence_timeout
+        )
+        self.mean_sweep_sf = self._tc_to_sf(self.mean_sweep_tc / scaling_factor, self.f)
+
     def process(self, result: a121.Result) -> ProcessorResult:
         frame = result.subframes[self.subsweep_index]
 
@@ -279,6 +374,21 @@ class Processor(ProcessorBase[ProcessorConfig, ProcessorResult]):
 
         inter = self._depth_filter(norm_lp_dev, self.depth_filter_length)
 
+        # Phase and amplitude weighting of inter-frame part
+
+        if self.inter_phase_boost:
+            if self.update_index == 0:
+                self.lp_mean_sweep_for_phase = mean_sweep
+
+            phase_and_amp_weight = self._calculate_phase_and_amp_weight(mean_sweep)
+
+            inter = inter * phase_and_amp_weight
+
+            sf = self._dynamic_sf(self.mean_sweep_sf, self.update_index)
+            self.lp_mean_sweep_for_phase = (
+                sf * self.lp_mean_sweep_for_phase + (1.0 - sf) * mean_sweep
+            )
+
         inter_presence_distance_index = int(np.argmax(inter))
         inter_presence_distance = self.distances[inter_presence_distance_index]
 
@@ -287,6 +397,21 @@ class Processor(ProcessorBase[ProcessorConfig, ProcessorResult]):
             sf * self.inter_presence_score + (1.0 - sf) * inter[inter_presence_distance_index]
         )
 
+        # Inter-frame presence timeout
+
+        if self.inter_frame_presence_timeout:
+            delta = self.inter_presence_score - self.previous_presence_score
+
+            if delta < 0:
+                self.negative_count += 1
+            else:
+                self.negative_count = 0
+
+            self._inter_presence_score_scaling()
+            self._mean_sweep_sf_scaling()
+
+            self.previous_presence_score = self.inter_presence_score
+
         # Presence distance - intra presence distance is prioritized due to faster reaction time
 
         if self.intra_presence_score > self.intra_threshold and self.intra_enable:
