@@ -4,6 +4,7 @@
 from __future__ import annotations
 
 import logging
+import pickle
 import queue
 import shutil
 import time
@@ -122,6 +123,8 @@ class AppModel(QObject):
     available_tagged_ports: list[Tuple[str, Optional[str]]]
     usb_connection_device: Optional[USBDevice]
     available_usb_devices: list[USBDevice]
+    overridden_baudrate: Optional[int]
+
     saveable_file: Optional[Path]
     autoconnect_enabled: bool
 
@@ -152,6 +155,8 @@ class AppModel(QObject):
         self.usb_connection_device = None
         self.available_tagged_ports = []
         self.available_usb_devices = []
+        self.overridden_baudrate = None
+
         self.saveable_file = None
         self.autoconnect_enabled = False
 
@@ -301,7 +306,7 @@ class AppModel(QObject):
 
     @classmethod
     def _handle_backend_serialized(
-        cls, *, generation: PluginGeneration, key: str, data: Optional[bytes]
+        cls, *, generation: PluginGeneration, key: str, data: Optional[dict]
     ) -> None:
         path = cls._get_plugin_config_path(generation, key)
 
@@ -309,7 +314,8 @@ class AppModel(QObject):
             path.unlink(missing_ok=True)
         else:
             path.parent.mkdir(parents=True, exist_ok=True)
-            path.write_bytes(data)
+            serialized_data = pickle.dumps(data, protocol=4)
+            path.write_bytes(serialized_data)
 
     def _update_saveable_file(self, path: Optional[Path]) -> None:
         if self.saveable_file is not None:
@@ -425,7 +431,9 @@ class AppModel(QObject):
         if self.connection_interface == ConnectionInterface.SOCKET:
             client_info = a121.ClientInfo(ip_address=self.socket_connection_ip)
         elif self.connection_interface == ConnectionInterface.SERIAL:
-            client_info = a121.ClientInfo(serial_port=self.serial_connection_port)
+            client_info = a121.ClientInfo(
+                serial_port=self.serial_connection_port, override_baudrate=self.overridden_baudrate
+            )
         elif self.connection_interface == ConnectionInterface.USB:
             client_info = a121.ClientInfo(usb_device=self.usb_connection_device)
         else:
@@ -495,6 +503,10 @@ class AppModel(QObject):
         self.autoconnect_enabled = autoconnect_enabled
         self.broadcast()
 
+    def set_overridden_baudrate(self, overridden_baudrate: Optional[int]) -> None:
+        self.overridden_baudrate = overridden_baudrate
+        self.broadcast()
+
     def _unload_current_plugin(self) -> None:
         log.debug("AppModel is unloading its current plugin")
         self.sig_load_plugin.emit(None)
@@ -511,10 +523,11 @@ class AppModel(QObject):
         config_path = self._get_plugin_config_path(generation, key)
         try:
             data = config_path.read_bytes()
+            cache = pickle.loads(data)
         except Exception:
             pass
         else:
-            self.put_backend_plugin_task("deserialize", {"data": data})
+            self.put_backend_plugin_task("_from_cache", {"data": cache})
 
     def load_plugin(self, plugin: Optional[PluginSpec]) -> None:
         log.debug(f"AppModel is loading the plugin {plugin}")
